﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace StdDemo
{
    public partial class DataMatrix : Form
    {
        private char[] data;
        private long size = 1;
        private bool result;
        private int RectWidth;
        private int RectHeight;
        private int BarcodeWidth;
        private int BarcodeHeight;
        private int PatternRows;
        private int PatternCols;

        public DataMatrix()
        {
            InitializeComponent();
        }

        private char GetElement(ref char[] Data, int RowsCount, int ColsCount, int RowIndex, int ColIndex)
        {
            return Data[(RowIndex - 1) * ColsCount + ColIndex - 1];
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            printBarcode.Print();
        }

        private void printBarcode_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            GraphicsUnit OldUnit = e.Graphics.PageUnit;
            e.Graphics.PageUnit = GraphicsUnit.Pixel;

            GetPatternData(e.Graphics);
            Render(e.Graphics, 100, 150);
            e.Graphics.PageUnit = OldUnit;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            Bitmap objBMP = new Bitmap(10, 10);
            Graphics g = Graphics.FromImage(objBMP);
            GetPatternData(g);
            objBMP.Dispose();

            SolidBrush sb = new SolidBrush(Color.White);
            const int PADDING_SIZE = 50;
            Rectangle rect = new Rectangle(0, 0, BarcodeWidth + PADDING_SIZE, BarcodeHeight + PADDING_SIZE);
            objBMP = new Bitmap(rect.Width, rect.Height);
            g = Graphics.FromImage(objBMP);
            g.FillRectangle(sb, rect);
            sb.Dispose();

            Render(g, PADDING_SIZE / 2, PADDING_SIZE / 2);
            objBMP.Save("c:\\temp\\DM.bmp", ImageFormat.Bmp);
            objBMP.Dispose();
        }

        private void GetPatternData(Graphics g)
        {
            // Message 
            objDM.Data = txtMessage.Text;

            // Mode
            objDM.Mode = (MW6.Barcode.Std.DataMatrix.enumMode)cbxMode.SelectedIndex;

            // Preferred Format
            objDM.PreferredFormat = (MW6.Barcode.Std.DataMatrix.enumPreferredFormat)cbxPreferredFormat.SelectedIndex;

            // Handle Tilde Character?
            objDM.HandleTilde = chkHandleTilde.Checked;

            // Get the buffer size
            PatternRows = 0;
            PatternCols = 0;

            result = objDM.GetPatternData(ref data, ref size, ref PatternRows, ref PatternCols);
            if (!result)
            {
                // Change the size of the array
                data = new char[size];

                // Obtain the pattern matrix data
                result = objDM.GetPatternData(ref data, ref size, ref PatternRows, ref PatternCols);
            }

            RectWidth       = (int)(g.DpiX * (float)System.Convert.ToDouble(cbxModuleSize.Text) / 2.54);
            RectHeight      = RectWidth;
            BarcodeWidth    = PatternCols * RectWidth;
            BarcodeHeight   = PatternRows * RectWidth;
        }

        private void Render(Graphics g, int StartX, int StartY)
        {
            int New_Rows = PatternRows;
            int New_Cols = PatternCols;
            
            SolidBrush sb = new SolidBrush(Color.Black);

            int Start_X = StartX;
            int Start_Y = StartY;
            int X = Start_X;
            int Y = Start_Y;
            bool IsBlackModule;
            int BlackModuleCount;

            for (int i = 1; i <= New_Rows; i++)
            {
                X = Start_X;
                IsBlackModule = true;
                BlackModuleCount = 0;
                for (int j = 1; j <= New_Cols; j++)
                {
                    char OneElement = GetElement(ref data, New_Rows, New_Cols, i, j);
                    if (j == 1)
                    {
                        if (OneElement == '1')
                        {
                            // A black module
                            IsBlackModule = true;
                            BlackModuleCount = 1;
                        }
                        else
                        {
                            // A white module (no drawing)
                            X = X + RectWidth;
                            IsBlackModule = false;
                            BlackModuleCount = 0;
                        }
                    }
                    else if (OneElement == '1' && IsBlackModule)
                    {
                        // Another black module
                        BlackModuleCount = BlackModuleCount + 1;
                    }
                    else if (OneElement == '1' && !IsBlackModule)
                    {
                        // A black module appears
                        IsBlackModule = true;
                        BlackModuleCount = 1;
                    }
                    else if (OneElement == '0' && IsBlackModule)
                    {
                        // Draw the continuous black modules
                        Rectangle rc = new Rectangle(X, Y, BlackModuleCount * RectWidth, RectHeight);

                        g.FillRectangle(sb, rc);
                        X = X + BlackModuleCount * RectWidth;
                        X = X + RectWidth;

                        IsBlackModule = false;
                    }
                    else if (OneElement == '0' && !IsBlackModule)
                    {
                        // A white module (no drawing)
                        X = X + RectWidth;
                    }
                }

                if (IsBlackModule)
                {
                    // Draw the continuous black modules
                    Rectangle rc = new Rectangle(X, Y, BlackModuleCount * RectWidth, RectHeight);

                    g.FillRectangle(sb, rc);
                }
                Y = Y + RectHeight;
            }

            sb.Dispose();
        }

        private void DataMatrix_Load(object sender, EventArgs e)
        {
            cbxMode.SelectedIndex               = 0;
            cbxModuleSize.SelectedIndex         = 2;
            cbxPreferredFormat.SelectedIndex    = 0;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        
    }
}
