﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace StdDemo
{
    public partial class OneDBarcodes : Form
    {
        private char[]  data;
        private long    size = 1;
        private bool    result;
        private int     RectWidth;
        private int     RectHeight;
        private int     BarcodeWidth;

        public OneDBarcodes()
        {
            InitializeComponent();
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            printBarcode.Print();
        }

        private void printBarcode_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            GraphicsUnit OldUnit = e.Graphics.PageUnit;
            e.Graphics.PageUnit = GraphicsUnit.Pixel;

            GetPatternData(e.Graphics);
            Render(e.Graphics, 100, 150);
            e.Graphics.PageUnit = OldUnit;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            Bitmap objBMP = new Bitmap(10, 10);
            Graphics g = Graphics.FromImage(objBMP);
            GetPatternData(g);
            objBMP.Dispose();

            SolidBrush sb = new SolidBrush(Color.White);
            const int PADDING_SIZE = 50;
            Rectangle rect = new Rectangle(0, 0, BarcodeWidth + PADDING_SIZE, RectHeight);
            objBMP = new Bitmap(rect.Width, rect.Height);
            g = Graphics.FromImage(objBMP);
            g.FillRectangle(sb, rect);
            sb.Dispose();
            
            Render(g, PADDING_SIZE / 2, 0);
            objBMP.Save("c:\\temp\\Barcode.bmp", ImageFormat.Bmp);
            objBMP.Dispose();
        }

        private void GetPatternData(Graphics g)
        {
            objOneDBarcode.Data = txtMessage.Text;
            objOneDBarcode.CheckDigit = chkAddCheckDigit.Checked;
            objOneDBarcode.SymbologyType = (MW6.Barcode.Std.Barcode.enumSymbologyType)(cbxTypes.SelectedIndex + 1);

            data = new char[1];
            size = 1;
            result = objOneDBarcode.GetPatternData(ref data, ref size);
            if (!result)
            {
                // Change the size of the array
                data = new char[size];

                // Obtain the pattern data
                result = objOneDBarcode.GetPatternData(ref data, ref size);
            }

            RectWidth = (int)(g.DpiX * (float)System.Convert.ToDouble(cbxNarrowBarWidth.Text) / 2.54);
            RectHeight = (int)(int)(g.DpiX * (float)System.Convert.ToDouble(cbxBarHeight.Text) / 2.54);

            BarcodeWidth = (int)(RectWidth * size);
        }

        private void Render(Graphics g, int StartX, int StartY)
        {
            SolidBrush sb = new SolidBrush(Color.Black);

            int Start_X = StartX;
            int Start_Y = StartY;
            int X = Start_X;
            int Y = Start_Y;
            bool IsBlackModule;
            int BlackModuleCount;

            X = Start_X;
            IsBlackModule = true;
            BlackModuleCount = 0;
            for (int i = 0; i < size; i++)
            {
                char OneElement = data[i];
                if (i == 0)
                {
                    if (OneElement == '1')
                    {
                        // A black module
                        IsBlackModule = true;
                        BlackModuleCount = 1;
                    }
                    else
                    {
                        // A white module (no drawing)
                        X = X + RectWidth;
                        IsBlackModule = false;
                        BlackModuleCount = 0;
                    }
                }
                else if (OneElement == '1' && IsBlackModule)
                {
                    // Another black module
                    BlackModuleCount = BlackModuleCount + 1;
                }
                else if (OneElement == '1' && !IsBlackModule)
                {
                    // A black module appears
                    IsBlackModule = true;
                    BlackModuleCount = 1;
                }
                else if (OneElement == '0' && IsBlackModule)
                {
                    // Draw the continuous black modules
                    Rectangle rc = new Rectangle(X, Y, BlackModuleCount * RectWidth, RectHeight);

                    g.FillRectangle(sb, rc);
                    X = X + BlackModuleCount * RectWidth;
                    X = X + RectWidth;

                    IsBlackModule = false;
                }
                else if (OneElement == '0' && !IsBlackModule)
                {
                    // A white module (no drawing)
                    X = X + RectWidth;
                }

                if (IsBlackModule)
                {
                    // Draw the continuous black modules
                    Rectangle rc = new Rectangle(X, Y, BlackModuleCount * RectWidth, RectHeight);

                    g.FillRectangle(sb, rc);
                }
            }

            sb.Dispose();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void OneDBarcodes_Load(object sender, EventArgs e)
        {
            cbxTypes.SelectedIndex          = 2;
            cbxNarrowBarWidth.SelectedIndex = 0;
            cbxBarHeight.SelectedIndex      = 0;
        }

        
    }
}
